"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildExecutor = void 0;
const tslib_1 = require("tslib");
const project_graph_1 = require("@nrwl/workspace/src/core/project-graph");
const operators_1 = require("rxjs/operators");
const rxjs_for_await_1 = require("rxjs-for-await");
const path_1 = require("path");
const node_config_1 = require("@nrwl/node/src/utils/node.config");
const normalize_1 = require("@nrwl/node/src/utils/normalize");
const run_webpack_1 = require("@nrwl/node/src/utils/run-webpack");
const buildable_libs_utils_1 = require("@nrwl/workspace/src/utilities/buildable-libs-utils");
const generate_package_json_1 = require("../../utils/generate-package-json");
const generate_package_json_lock_file_1 = require("../../utils/generate-package-json-lock-file");
try {
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    require('dotenv').config();
}
catch (_a) {
    // do nothing
}
function buildExecutor(rawOptions, context) {
    return (0, tslib_1.__asyncGenerator)(this, arguments, function* buildExecutor_1() {
        const { sourceRoot, root } = context.workspace.projects[context.projectName];
        if (!sourceRoot) {
            throw new Error(`${context.projectName} does not have a sourceRoot.`);
        }
        if (!root) {
            throw new Error(`${context.projectName} does not have a root.`);
        }
        const options = (0, normalize_1.normalizeBuildOptions)(rawOptions, context.root, sourceRoot, root);
        if (!options.buildLibsFromSource) {
            const { target, dependencies } = (0, buildable_libs_utils_1.calculateProjectDependencies)((0, project_graph_1.readCachedProjectGraph)(), context.root, context.projectName, context.targetName, context.configurationName);
            options.tsConfig = (0, buildable_libs_utils_1.createTmpTsConfig)(options.tsConfig, context.root, target.data.root, dependencies);
            if (!(0, buildable_libs_utils_1.checkDependentProjectsHaveBeenBuilt)(context.root, context.projectName, context.targetName, dependencies)) {
                return yield (0, tslib_1.__await)({ success: false });
            }
        }
        const config = options.webpackConfig.reduce((currentConfig, plugin) => {
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            return require(plugin)(currentConfig, {
                options,
                configuration: context.configurationName
            });
        }, (0, node_config_1.getNodeWebpackConfig)(options));
        return yield (0, tslib_1.__await)(yield (0, tslib_1.__await)(yield* (0, tslib_1.__asyncDelegator)((0, tslib_1.__asyncValues)((0, rxjs_for_await_1.eachValueFrom)((0, run_webpack_1.runWebpack)(config).pipe((0, operators_1.tap)((stats) => {
            console.info(stats.toString(config.stats));
        }), (0, operators_1.map)((stats) => {
            return {
                success: !stats.hasErrors(),
                outfile: (0, path_1.resolve)(context.root, options.outputPath, options.outputFileName)
            };
        }), (0, operators_1.tap)(({ outfile }) => ((0, generate_package_json_1.generatePackageJson)(context.projectName, (0, project_graph_1.readCachedProjectGraph)(), options, outfile, context.root))), (0, operators_1.tap)(() => {
            if (rawOptions.generateLockFile) {
                (0, generate_package_json_lock_file_1.generatePackageJsonLockFile)(context.root, context.projectName, options);
            }
        }), (0, operators_1.tap)(() => ({
            success: true
        }))))))));
    });
}
exports.buildExecutor = buildExecutor;
exports.default = buildExecutor;
//# sourceMappingURL=build.impl.js.map